/***********************************************************************
 * File: aggregate_test.c
 * 	(Extracted from
 *	   "$Id: aggregate_test.c,v 1.6 2009/05/22 16:07:00 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This C source file implements test program for `aggregate()' API.
 *
 * Copyright (C) 2009 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

#include "aggregate.h" /* definitions of `aggregate' API-s */

/* test program for `aggregate.c' API-s */
int main(int /*@unused@*/ ac, char /*@unused@*/ **av)
{
	static char	*codes	= "z+-. 09";
	static char	*strs[]	=	{
							  "-9999"
							, "-999 "
							, "-99.1"
							, "-99.0"
							, "-99  "
							, "-9.1 "
							, "-9.01"
							, "-9.0 "
							, "-0.1 "
							, "-.01 "
							, "-.001"
							, "-.000"
							, ""
							, "0.000"
							, "00.00"
							, "  0.0"
							, " .001"
							, "+.010"
							, " +.1 "
							, "   9."
							, " 9.01"
							, "  9.1"
							, "  +99"
							, " 99.1"
							, " 999 "
							, " 9999"
							};
	static Aggregate_e_opcode	opcodes[] = {
											  Aggregate_e_opcode_add
											, Aggregate_e_opcode_sub
											, Aggregate_e_opcode_mul
											,
											Aggregate_e_opcode_div
											};
	static char	*signs[]	= { "-", "" };
	static char	*numerals[] =	{
								 "9999999999999999999999999"
								, "9"
								, ""
								};
	static char	*sums[] =	{ "1", "2", "3", "4", "5"
							, "6", "7", "8", "9", "10"
							};
	#define		Aggregate_numerals_sz	(1 + 1 + 25 + 1 + 25 + 1)
	#define		N_numerals				(2)
	char		a_str[6], b_str[6]
				, numeral_strs[N_numerals][Aggregate_numerals_sz]
				, *numeral_strps[N_numerals]
				, *retstrp
				;
	ssize_t		scale;
	size_t		len, i, j, k, l, m, n, o, h, gprof_cnt, gprof_iter_max;
	int			*retval, result;

	for (gprof_iter_max = (size_t)((ac == 2 && strcmp(av[1], "-P") == 0)?
																	100: 1
									)
		, gprof_cnt = 0
	; gprof_cnt < gprof_iter_max
	; ++gprof_cnt
	) {	/* iterate the unit test for gprof profiling... */
	fprintf(stderr, "{{{ tests for compare_numbers()...\n")
	; (void)fflush(stderr)
	;
	len	= strlen(codes);
	for (i = 0; i < len; ++i)
	for (j = 0; j < len; ++j)
	for (k = 0; k < len; ++k)
	for (l = 0; l < len; ++l)
	{
		memset(a_str, (int)'\0', sizeof(a_str))
		; memset(b_str, (int)'\0', sizeof(b_str))
		; a_str[0] = b_str[0]	= codes[i]
		; a_str[1] = b_str[1]	= codes[j]
		; a_str[2] = b_str[2]	= codes[k]
		; a_str[3] = b_str[3]	= codes[l]
		; result	= -9;
		; errno		= 0
		; retval	= compare_numbers((const char *)a_str
								, (const char *)b_str
								, &result
								);
		fprintf(stderr
		,	"a_str[]=\"%s\", b_str=\"%s\", "
			"compare_numbers() returns %p, result=%d, errno=%d.\n"
		,				a_str,		b_str
		,							(void *)retval,	result, errno
		); (void)fflush(stderr)
		;
	}

	fprintf(stderr, "\n"); (void)fflush(stderr);

	for (i = 0; i < (sizeof(strs) /sizeof(strs[0])); ++i)
	for (j = 0; j < (sizeof(strs) / sizeof(strs[0])); ++j)
	{
		result		= -9;
		; errno		= 0
		; retval	= compare_numbers(strs[i], strs[j], &result);
		fprintf(stderr
		,	"strs[%lu]=\"%s\", strs[%lu]=\"%s\", "
			"compare_numbers() returns %p, result=%d, errno=%d.\n"
		,		(unsigned long)i, strs[i],		(unsigned long)j, strs[j]
		,							(void *)retval,	result, errno
		); (void)fflush(stderr)
		;
	}
	fprintf(stderr, "}}} finished tests for compare_numbers()\n")
	; (void)fflush(stderr)
	;

	fprintf(stderr, "\n"); (void)fflush(stderr);

	fprintf(stderr, "{{{ tests for aggregate_numbers()...\n")
	; (void)fflush(stderr)
	;
	for (i = 0; i < (sizeof(opcodes) / sizeof(opcodes[0])); ++i)
	for (j = 0; j < (sizeof(signs) / sizeof(signs[0])); ++j)
	for (k = 0; k < (sizeof(numerals) / sizeof(numerals[0])); ++k)
	for (l = 0; l < (sizeof(numerals) / sizeof(numerals[0])); ++l)
	for (m = 0; m < (sizeof(signs) / sizeof(signs[0])); ++m)
	for (n = 0; n < (sizeof(numerals) / sizeof(numerals[0])); ++n)
	for (o = 0; o < (sizeof(numerals) / sizeof(numerals[0])); ++o)
	for (h = 0; h < 3; ++h) {
		(void)snprintf(numeral_strs[0], Aggregate_numerals_sz, "%s%s.%s"
				, signs[j], numerals[k], numerals[l]
				)
		; (void)snprintf(numeral_strs[1], Aggregate_numerals_sz, "%s%s.%s"
				, signs[m], numerals[n], numerals[o]
				)
		; numeral_strps[0] = numeral_strs[0]
		; numeral_strps[1] = numeral_strs[1]
		;
		if (h == 0) {
			scale	= (ssize_t)(
							(opcodes[i] == Aggregate_e_opcode_add)?		25
							: (opcodes[i] == Aggregate_e_opcode_sub)?	25
							:											50
							);
		}
		else if (h == 1) {
			scale	= 10;
		}
		else {
			scale	= -1;
		}
		fprintf(stderr
				,	"%s\n"
					"%s\n"
					"%s\t(scale=%ld)\n"
			  	, numeral_strps[0]
				, ((opcodes[i] == Aggregate_e_opcode_add)?		"+"
					: ((opcodes[i] == Aggregate_e_opcode_sub)?	"-"
					: ((opcodes[i] == Aggregate_e_opcode_mul)?	"*"
					:											"/"
					)))
				, numeral_strps[1],	(long)scale
			  ); (void)fflush(stderr);
		retstrp	= aggregate_numbers(
						  opcodes[i]
						, (const char **)numeral_strps
						, N_numerals
						, scale
						);
		if (retstrp == NULL) {
			fprintf(stderr, "***ERROR: errno=%d\n", errno);
		}
		else {
			fprintf(stderr, "%s\n", retstrp);
			(void)destroy_aggregate_numbers(retstrp);
		}
		fprintf(stderr, "--------------------------------------\n")
		; (void)fflush(stderr)
		;
	}

	fprintf(stderr, "\n"); (void)fflush(stderr);

	for (k = 0; k < 2; ++k)
	for (i = 0; i < (sizeof(opcodes)/ sizeof(opcodes[0])); ++i)
	for (j = 0; j < (sizeof(sums) / sizeof(sums[0])); ++j) {
		fprintf(stderr, "%s", sums[0]);
		if (k == 0) {
			scale	= (ssize_t)(
							(opcodes[i] == Aggregate_e_opcode_add)?		25
							: (opcodes[i] == Aggregate_e_opcode_sub)?	25
							:											50
							);
		}
		else {
			scale	= -1;
		}
		for (l = 1; l < j; ++l) {
			fprintf(stderr, "%s%s"
					, ((opcodes[i] == Aggregate_e_opcode_add)?		"+"
						: ((opcodes[i] == Aggregate_e_opcode_sub)?	"-"
						: ((opcodes[i] == Aggregate_e_opcode_mul)?	"*"
						:											"/"
						)))
					, sums[l]
					);
		}
		fprintf(stderr, "\t(scale=%ld)\n", (long)scale)
		; (void)fflush(stderr)
		;
		retstrp	= aggregate_numbers(
						  opcodes[i]
						, (const char **)sums
						, j
						, scale
						);
		if (retstrp == NULL) {
			fprintf(stderr, "***ERROR: errno=%d\n", errno);
		}
		else {
			fprintf(stderr, "%s\n", retstrp);
			(void)destroy_aggregate_numbers(retstrp);
		}
		fprintf(stderr, "-------------------------------------\n")
		; (void)fflush(stderr)
		;
	}
	fprintf(stderr, "}}} finished tests for aggregate_numbers()\n")
	; (void)fflush(stderr)
	;

	fprintf(stderr, "\n"); (void)fflush(stderr);
	} /* iterate the unit test for gprof profiling... */

	exit(EXIT_SUCCESS); /* exit test program for `abacus()' API */
}
